/* eslint-disable jsx-a11y/no-static-element-interactions */
/* eslint-disable react/no-array-index-key */
/* eslint-disable jsx-a11y/click-events-have-key-events */
/* eslint-disable react/jsx-filename-extension */
/* eslint-disable react/forbid-prop-types */
import React from 'react'
import cx from 'classnames'
import PropTypes from 'prop-types'
import isEmpty from 'is-empty'
import { noop, getInputValue } from './utils'
import { Tag } from './Tag'
import { DEFAULT_LABELKEY } from './constants'

const getSelectedValue = ({ selected, multiple, labelKey, onMenuItemRender }) => {
  if (multiple) return ''
  return getInputValue(selected[0], { labelKey, onMenuItemRender }) || ''
}

const InputWrapper = props => {
  let {
    onFocus,
    inputClasses,
    icons,
    inputRef,
    placeHolder,
    disabled,
    loading,
    multiple,
    selected = [],
    labelKey,
    onMenuItemRender,
    onRemove,
    open,
    extraProps,
    ...inputProps
  } = props


  let hasLeftIcon = !isEmpty(icons.left.component)
  let hasRightIcon = !isEmpty(icons.right.component)

  let value = isEmpty(selected) ? '' : getSelectedValue(props)

  return (
    <div className='pk-select-input-wrapper'>
      {
        hasLeftIcon ? (
          <span role='button' tabIndex='-1' className={cx('pk-select-input-icon icon-left', icons.left.additionalClasses)} onClick={() => icons.left.onClick()}>
            {icons.left.component}
          </span>
        )
          :
          null
      }
      {
        disabled ? (
          <input
            className={cx('pk-select-input')}
            defaultValue={value}
            disabled
          />
        )
          : (
            <>
              {
                !multiple ? (
                  <input
                    className={cx('pk-select-input read-only', inputClasses, { 'pk-select-has-focus': open, 'has-left-icon': hasLeftIcon, 'has-right-icon': hasRightIcon })}
                    value={value}
                    ref={inputRef}
                    placeholder={placeHolder}
                    onFocus={onFocus}
                    readOnly
                    {...inputProps}
                    {...extraProps}
                  />
                )
                  : (
                    <div
                      ref={inputRef}
                      className={cx('pk-select-input', inputClasses, { 'pk-select-has-focus': open, 'has-left-icon': hasLeftIcon, 'has-right-icon': hasRightIcon })}
                    >
                      {
                        selected.map((_selected, i) => (
                          <Tag closeable onClose={() => onRemove(_selected)} key={i}>
                            {getInputValue(_selected, { labelKey, onMenuItemRender })}
                          </Tag>
                        ))
                      }
                      <input
                        className='pk-select-multiple-input cursor-pointer'
                        onFocus={onFocus}
                        placeholder={placeHolder}
                        readOnly
                        {...inputProps}
                        {...extraProps}
                      />
                    </div>
                  )
              }
            </>
          )}
      <>
        {
          loading ? (
            <div className='pk-select-loader-wrapper'>
              <div className='pk-select-loader'>
                <svg viewBox='22 22 44 44'>
                  <circle className='path' cx='44' cy='44' r='19.5' fill='none' strokeWidth='4' />
                </svg>
              </div>
            </div>
          )
            :
            hasRightIcon ? (
              <span role='button' tabIndex='-1' className={cx('pk-select-input-icon icon-right', icons.right.additionalClasses)} onClick={() => icons.right.onClick()} ref={icons.right.iconRef}>
                {icons.right.component}
              </span>
            )
              :
              null
        }
      </>
    </div>
  )
}

const iconProps = {
  component: PropTypes.any,
  onClick: PropTypes.func,
  additionalClasses: PropTypes.string,
  iconRef: PropTypes.func
}

InputWrapper.propTypes = {
  onFocus: PropTypes.func,
  disabled: PropTypes.bool,
  inputClasses: PropTypes.string,
  icons: PropTypes.shape({
    left: PropTypes.shape(iconProps),
    right: PropTypes.shape(iconProps),
  }),
  inputRef: PropTypes.func,
  placeHolder: PropTypes.string,
  loading: PropTypes.bool,
  multiple: PropTypes.bool,
  selected: PropTypes.array,
  labelKey: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  onMenuItemRender: PropTypes.oneOfType([PropTypes.bool, PropTypes.func]),
  onRemove: PropTypes.func,
  open: PropTypes.bool,
  extraProps: PropTypes.object
}

InputWrapper.defaultProps = {
  onFocus: noop,
  disabled: false,
  inputClasses: '',
  icons: {},
  inputRef: noop,
  loading: false,
  placeHolder: '',
  multiple: false,
  selected: [],
  labelKey: DEFAULT_LABELKEY,
  onMenuItemRender: false,
  onRemove: noop,
  open: false,
  extraProps: {}
}

export default InputWrapper
