/* eslint-disable jsx-a11y/no-autofocus */
/* eslint-disable react/default-props-match-prop-types */
/* eslint-disable react/forbid-prop-types */
/* eslint-disable react/require-default-props */
/* eslint-disable react/no-array-index-key */
/* eslint-disable react/jsx-filename-extension */
import React, { useState } from 'react'
import cx from 'classnames'
import PropTypes from 'prop-types'
import { MAXIMUM_DROPDOWN_HEIGHT, DEFAULT_LABELKEY } from './constants'
import MenuItem from './MenuItem'
import { getLabelKey, noop } from './utils'

const Menu = (props) => {
  const displayName = 'menu-item'
  const [_menu, setMenu] = useState(null)
  const {
    maxHeight, options, selected, activeIndex, open, loading, labelKey,
    isDirty, style, menuRef, onMenuClick, autoScroll, disableAutoScroll, animate,
    searchInputProps, searchable, scrollableAreaRef
  } = props
  const isSelected = option => {
    const _selected = [...selected].shift()
    return _selected && option[getLabelKey(labelKey, option)] === _selected[getLabelKey(labelKey, option)]
  }
  const { dropup, ...menuPositionStyle } = style
  const handleMenuScroll = () => {
    disableAutoScroll()
  }
  return (
    <div
      className={cx('pk-select--dropdown', { open })}
      style={{ ...menuPositionStyle }}
      ref={menuRef}
    >
      <div className={cx('pk-select--transition', { 'slide-up': (animate && dropup), 'slide-down': (animate && !dropup) })}>
        <div className={cx('pk-select--popup', { 'has-shadow': open, 'pt-4': !searchable || dropup, 'pb-4': !searchable || !dropup, 'column-reverse': dropup })}>
          {
            searchable && (
              <div className={cx('pk-select--search-input-wrapper', { 'mb-8': !dropup, 'mt-8': dropup })}>
                <input
                  type='text'
                  autoComplete='off'
                  autoFocus
                  className={cx('pk-select--search-input', { dropup })}
                  placeholder='Search...'
                  tabIndex='-1'
                  {...searchInputProps}
                />
              </div>
            )
          }
          <div
            className='scroll-y'
            ref={(m) => {
              setMenu(m)
              scrollableAreaRef(m)
            }}
            onScroll={handleMenuScroll}
            style={{ maxHeight }}
          >
            {
              loading ?
                <div className='pk-select--dropdown_item item-disabled'>Searching...</div>
                :
                options.length === 0 ? (
                  <div className='pk-select--dropdown_item item-disabled'>
                    {
                      (isDirty || !searchable) ? 'No result found' : 'Type to search...'
                    }
                  </div>
                )
                  :
                  options.map((option, i) => (
                    <MenuItem
                      key={`${displayName}-${i}`}
                      label={option.__label}
                      activeIndex={activeIndex}
                      position={i}
                      selected={isSelected(option)}
                      menuContainer={_menu}
                      onClick={() => onMenuClick(option, i)}
                      autoScroll={autoScroll}
                    />
                  ))
            }
          </div>
        </div>
      </div>
    </div>
  )
}

Menu.propTypes = {
  options: PropTypes.array.isRequired,
  items: PropTypes.array.isRequired,
  selected: PropTypes.array.isRequired,
  activeIndex: PropTypes.number,
  open: PropTypes.bool,
  onExit: PropTypes.func,
  onExiting: PropTypes.func,
  onEnter: PropTypes.func,
  onEntering: PropTypes.func,
  maxHeight: PropTypes.number,
  style: PropTypes.object,
  menuRef: PropTypes.func,
  onMenuClick: PropTypes.func,
  autoScroll: PropTypes.bool,
  disableAutoScroll: PropTypes.func,
  loading: PropTypes.bool,
  isDirty: PropTypes.bool,
  animate: PropTypes.bool,
  labelKey: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  onMenuItemRender: PropTypes.oneOfType([PropTypes.bool, PropTypes.func]),
  searchable: PropTypes.bool,
  searchInputProps: PropTypes.object,
  scrollableAreaRef: PropTypes.func
}

Menu.defaultProps = {
  options: [],
  items: [],
  selected: [],
  activeIndex: -1,
  open: false,
  onExiting: noop,
  onExit: noop,
  onEntering: noop,
  onEnter: noop,
  maxHeight: MAXIMUM_DROPDOWN_HEIGHT,
  style: {},
  menuRef: noop,
  onMenuClick: noop,
  autoScroll: false,
  disableAutoScroll: noop,
  loading: false,
  isDirty: false,
  labelKey: DEFAULT_LABELKEY,
  searchable: false,
  searchInputProps: {},
  scrollableAreaRef: noop
}

export default Menu