/* eslint-disable react/destructuring-assignment */
/* eslint-disable react/jsx-filename-extension */
/* eslint-disable react/default-props-match-prop-types */
/* eslint-disable react/forbid-prop-types */
/* eslint-disable react/require-default-props */
import React, { PureComponent } from 'react'
import PropTypes from 'prop-types'
import Portal from 'react-overlays/Portal'
import isEmpty from 'is-empty'
import cx from 'classnames'
import {
  DEFAULT_LABELKEY, SELECT_HEIGHT, SELECT_WIDTH,
  MAXIMUM_DROPDOWN_HEIGHT, MINIMUM_DROPDOWN_WIDTH,
  UP, DOWN, ESC, RETURN, TAB, DROPDOWN_ITEM_HEIGHT, ACTUAL_VALUE
} from './constants'
import InputWrapper from './InputWrapper'
import Menu from './Menu'
import {
  noop, getStateFromProps, constructMenuProps,
  getUpdatedActiveIndex, optionsMap, filterResultsFromOptions, getInputValue
} from './utils'
import { isEqual, debounce } from './helpers'
import ArrowUpDown from './icons/arrow-up-down'

const getMenuHeight = (maxDropdownHeight, length) => Math.min(maxDropdownHeight, (length ? (length * DROPDOWN_ITEM_HEIGHT) + 15 : DROPDOWN_ITEM_HEIGHT))

const getOffset = (el) => {
  let rect = el.getBoundingClientRect().toJSON()
  let scrollLeft = window.pageXOffset || document.documentElement.scrollLeft
  let scrollTop = window.pageYOffset || document.documentElement.scrollTop
  return {
    ...rect,
    top: rect.top + scrollTop,
    left: rect.left + scrollLeft
  }
}

const getActiveIndex = (options = [], selected = false, props) => {
  let { labelKey, defaultFirstItemSelected, async, onMenuItemRender } = props
  if (!selected) {
    return defaultFirstItemSelected && options.length ? 0 : -1
  }
  if (async || props.multiple) {
    return defaultFirstItemSelected ? 0 : -1
  }
  return options.findIndex(option => getInputValue(option, { labelKey, onMenuItemRender }) === getInputValue([...selected].shift(), { labelKey, onMenuItemRender }))
}

class Select extends PureComponent {
  _input = null
  input = null
  _dropdownScrollableArea = null
  state = getStateFromProps(this.props)

  componentDidMount() {
    this._listenResizeEvent()
  }

  UNSAFE_componentWillReceiveProps(nextProps) {
    let { labelKey, multiple, options, onMenuItemRender } = nextProps
    let selected = optionsMap([...nextProps.selected], { labelKey, onMenuItemRender })
    let nextOptions = optionsMap([...options], { labelKey, onMenuItemRender })
    let nextState = {}
    let { options: currentOptions, selected: currentSelected, value } = this.state
    let { multiple: currentMultiple } = this.props
    if (!isEqual(nextOptions, currentOptions)) {
      nextState.options = nextOptions
      nextState.results = filterResultsFromOptions([...nextOptions], {
        ...nextProps,
        value
      })
      nextState.activeIndex = getActiveIndex([...nextState.results], selected.length && selected[0], nextProps)
      nextState.activeItem = nextState.activeIndex === -1 ? null : nextState.results[nextState.activeIndex]
    }

    if (multiple !== currentMultiple) {
      nextState.value = ''
    }

    // If new selections are passed via props, treat as a controlled input.
    if (selected && !isEqual(selected, currentSelected)) {
      nextState.selected = [...selected]

      if (multiple) {
        nextState.results = filterResultsFromOptions([...nextOptions], {
          ...nextProps,
          value: '',
        })
        nextState.value = ''
        this.setState(nextState)
        return
      }
      nextState.value = selected.length ? getInputValue(selected[0], { labelKey, onMenuItemRender }) : ''
    }

    // Truncate selections when in single-select mode.
    let newSelected = selected || currentSelected
    if (!multiple && newSelected.length > 1) {
      newSelected = newSelected.slice(0, 1)
      nextState.selected = [...newSelected]
      nextState.value = getInputValue(newSelected[0], { labelKey, onMenuItemRender })
    }

    this.setState(nextState)
  }

  componentDidUpdate(prevProps, prevState) {
    let { value, options } = this.state
    if (prevState.value !== value) {
      let { defaultFirstItemSelected } = this.props
      let results = filterResultsFromOptions([...options], {
        ...this.props,
        value
      })
      let activeIndex = defaultFirstItemSelected ? 0 : -1
      let activeItem = defaultFirstItemSelected ? results[0] : null
      // eslint-disable-next-line react/no-did-update-set-state
      this.setState({
        results,
        activeItem,
        activeIndex
      }, () => {
        if (this._dropdownScrollableArea) {
          this._dropdownScrollableArea.scrollTop = 0
        }
      })
    }
  }

  componentWillUnmount() {
    this._revokeResizeListener()
  }

  // eslint-disable-next-line react/sort-comp
  _setInputRef = input => {
    this.input = input
    this._setMenuPosition()
  }

  _setBackgroundInputRef = input => {
    this._input = input
  }

  _setDropdownScrollableArea = dropdownScrollArea => {
    this._dropdownScrollableArea = dropdownScrollArea
  }

  _setMenuRef = menu => {
    this.menu = menu
  }

  _checkMenuPosition = () => {
    let nextMenuPosition = this._getMenuPosition()
    let { menuPosition } = this.state
    if (!isEqual(nextMenuPosition, menuPosition)) {
      this._setMenuPosition(nextMenuPosition)
    }
  }

  _getMenuPosition = () => {
    if (!this.input) {
      return
    }
    let { maxDropdownHeight, dropup } = this.props
    let { results } = this.state
    let menuHeight = getMenuHeight(maxDropdownHeight, results.length)
    let body = document.documentElement || document.body
    let position = getOffset(this.input)
    let bottom = 'auto'
    let top = position.top + position.height + 2
    let _dropup = false
    if (dropup || ((position.bottom + menuHeight) > (body.scrollHeight - body.scrollTop))) {
      top = 'auto'
      bottom = (body.scrollHeight - body.scrollTop - position.top + 2)
      _dropup = true
    }
    return {
      top,
      bottom,
      left: position.left,
      width: position.width,
      dropup: _dropup
    }
  }

  _setMenuPosition = (menuPosition = this._getMenuPosition()) => {
    this.setState(prevState => ({
      ...prevState,
      menuPosition
    }))
  }

  _handleInputChange = e => {
    e.persist()
    let value = e.currentTarget.value
    let clearSelection = !this.props.multiple && this.state.selected.length
    this.setState((prevState) => {
      return {
        isDirty: true,
        selected: clearSelection ? [] : prevState.selected,
        value,
      }
    }, () => {
      this.props.onInputChange && this.props.onInputChange(value, e)
      clearSelection && this.props.onChange([])
    })
  }

  _handleActiveIndexChange = (activeIndex) => {
    this.setState((state) => ({
      activeIndex,
      activeItem: activeIndex === -1 ? null : state.results[activeIndex],
    }))
  }

  _handleKeyDown = (e) => {
    let { activeItem } = this.state
    let { multiple, searchable } = this.props
    let input = multiple ? this._input : this.input

    // Skip most actions when the menu is hidden.
    if (!this._isMenuOpen()) {
      if (e.keyCode === UP || e.keyCode === DOWN) {
        this._openMenu()
      }

      return
    }

    switch (e.keyCode) {
      case UP:
      case DOWN:
        // Prevent input cursor from going to the beginning when pressing up.
        e.preventDefault()
        this._enableAutoScroll()
        this._handleActiveIndexChange(getUpdatedActiveIndex(
          this.state.activeIndex,
          e.keyCode,
          this.state.results
        ))
        break
      case RETURN:
        // Prevent form submission while menu is open.
        e.preventDefault()
        activeItem && this._onMenuClick(activeItem)
        break
      case ESC:
      case TAB:
        // ESC simply hides the menu. TAB will blur the input and move focus to
        // the next item hide the menu so it doesn't gain focus.
        searchable && input.focus()
        this._closeMenu()
        break
      default:
        break
    }
  }

  _onClick = e => {
    e.persist()
    let { onInputClick } = this.props

    if (this._hasFocus() && !this._isMenuOpen()) {
      this._openMenu()
    }

    onInputClick && onInputClick()
  }

  _onFocus = e => {
    e.persist()
    if (!this._hasFocus()) {
      this._setFocus(true)
      this._openMenu()
    } else {
      !this._isMenuOpen() && this._openMenu()
    }
    this.props.onInputFocus && this.props.onInputFocus(e)
  }

  _onBlur = e => {
    e.persist()
    this._hasFocus() && this._setFocus(false)
    this.props.onInputBlur && this.props.onInputBlur(e)
  }

  _onMenuClick = option => {
    let {
      [ACTUAL_VALUE]: selected
    } = option
    if (this.props.multiple) {
      let prevSelected = [...this.state.selected].map(s => s[ACTUAL_VALUE])
      selected = [...prevSelected, selected]
    } else {
      selected = [selected]
    }
    this.props.onChange && this.props.onChange(selected)
    this._closeMenu()
  }

  _onRemoveSelected = option => {
    if (!this.props.multiple) return
    let { labelKey, onMenuItemRender } = this.props
    let filterOption = { labelKey, onMenuItemRender }
    let selected = [...this.state.selected].filter(s => getInputValue(s, filterOption) !== getInputValue(option, filterOption)).map(ms => ms[ACTUAL_VALUE])
    this.props.onChange && this.props.onChange(selected)
    this._isMenuOpen() && this._closeMenu()
  }

  _hasFocus = () => this.state.focus

  _isMenuOpen = () => this.state.open

  _setFocus = focus => {
    this.setState({
      focus
    })
  }

  _triggerOpen = () => {
    if (!this._isMenuOpen()) {
      this.input && this.input.focus()
      this._openMenu()
    } else {
      this.input && this.input.blur()
      this._closeMenu()
    }
  }

  _openMenu = () => {
    this._checkMenuPosition()
    this._enableAutoScroll()
    this.setState(state => {
      let nextState = {
        open: true,
      }
      if (!state.searchable) {
        nextState.activeIndex = getActiveIndex(state.results, state.selected.length && state.selected, this.props)
      } else {
        nextState.activeIndex = this.props.defaultFirstItemSelected ? 0 : -1
      }
      nextState.activeItem = nextState.activeIndex === -1 ? null : state.results[nextState.activeIndex]
      return nextState
    }, () => {
      setTimeout(() => {
        this._listenClickEvent()
      }, 100)
    })
  }

  _closeMenu = () => {
    this._disableAutoScroll()
    this.setState({
      activeIndex: -1,
      activeItem: null,
      open: false,
      focus: false
    }, () => {
      setTimeout(() => {
        this._revokeClickListener()
        let input = this.props.multiple ? this._input : this.input
        input && input.blur()
      }, 100)
    })
  }

  _listenResizeEvent = () => window.addEventListener('resize', this._handleResize)

  _revokeResizeListener = () => window.removeEventListener('resize', this._handleResize)

  _listenClickEvent = () => {
    setTimeout(() => {
      if (this._isMenuOpen()) {
        document.addEventListener('click', this._handleOutsideClick, false)
        document.addEventListener('scroll', this._handlePageScroll, true)
      }
    }, 100)
  }

  _revokeClickListener = () => {
    document.removeEventListener('click', this._handleOutsideClick, false)
    document.addEventListener('scroll', this._handlePageScroll, true)
  }

  _handleOutsideClick = (event) => {
    if (this._isMenuOpen() && (this.menu && !this.menu.contains(event.target)) && this.props.closeOnOutsideClick) {
      this._closeMenu()
    }
  }

  _handleResize = () => {
    this._recomputeMenuPostition()
  }

  _handlePageScroll = (e) => {
    if (this.menu && !this.menu.contains(e.target)) {
      this._closeMenu()
    }
  }

  // eslint-disable-next-line react/sort-comp
  _recomputeMenuPostition = debounce(() => {
    this._setMenuPosition()
  }, 500)

  _autoScrollEnabled = () => this.state.autoScroll

  _enableAutoScroll = () => {
    if (!this._autoScrollEnabled()) {
      this.setState({
        autoScroll: true
      })
    }
  }

  _disableAutoScroll = () => {
    if (this._autoScrollEnabled()) {
      this.setState({
        autoScroll: false
      })
    }
  }

  render() {
    let {
      id,
      width,
      height,
      inputProps,
      placeHolder,
      searchable,
      loading,
      icons,
      onMenuItemRender,
      multiple,
      labelKey,
      containerClass = ''
    } = this.props

    let {
      value,
      menuPosition,
      open,
      selected
    } = this.state

    let mergedStateAndProps = {
      ...this.props,
      ...this.state,
    }

    let rightIcon = {
      component: isEmpty(icons.right) ? <ArrowUpDown /> : icons.right,
      onClick: this._triggerOpen,
      additionalClasses: 'is-clickable'
    }

    let leftIcon = {
      component: icons.left ? icons.left : null
    }

    let containerStyle = multiple ? { minHeight: height, height: 'auto', width } : { height, width }

    let extraProps = searchable ? {} : {
      onKeyDown: this._handleKeyDown,
      onClick: this._onFocus
    }

    if (multiple) {
      extraProps.ref = this._setBackgroundInputRef
    }

    return (
      <div className={cx('pk-select', containerClass, { 'pk-select--multiple': multiple })} id={id} style={containerStyle}>
        <InputWrapper
          {...inputProps}
          placeHolder={placeHolder ? placeHolder : (searchable ? 'Search...' : 'Select...')}
          onFocus={this._onFocus}
          inputRef={this._setInputRef}
          loading={loading}
          icons={{
            left: leftIcon,
            right: rightIcon
          }}
          multiple={multiple}
          selected={selected}
          labelKey={labelKey}
          onMenuItemRender={onMenuItemRender}
          onRemove={this._onRemoveSelected}
          open={open}
          extraProps={extraProps}
        />
        <Portal container={document.body}>
          {
            open && (
              <Menu
                {...constructMenuProps(mergedStateAndProps)}
                disableAutoScroll={this._disableAutoScroll}
                menuRef={this._setMenuRef}
                scrollableAreaRef={this._setDropdownScrollableArea}
                style={menuPosition}
                open={open}
                onMenuClick={this._onMenuClick}
                onMenuItemRender={onMenuItemRender}
                searchable={searchable}
                searchInputProps={{
                  value,
                  onChange: searchable ? this._handleInputChange : noop,
                  onKeyDown: this._handleKeyDown
                }}
              />
            )
          }
        </Portal>
      </div>
    )
  }
}

Select.propTypes = {
  options: PropTypes.array.isRequired,
  selected: PropTypes.array.isRequired,
  multiple: PropTypes.bool.isRequired,
  id: PropTypes.string.isRequired,
  onChange: PropTypes.func.isRequired,
  labelKey: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  searchable: PropTypes.bool,
  defaultFirstItemSelected: PropTypes.bool,
  onInputChange: PropTypes.func,
  onInputFocus: PropTypes.func,
  onInputBlur: PropTypes.func,
  minLength: PropTypes.number,
  inputProps: PropTypes.object,
  placeHolder: PropTypes.string,
  filterBy: PropTypes.oneOfType([PropTypes.array, PropTypes.func]),
  closeOnOutsideClick: PropTypes.bool,
  width: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  height: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  maxDropdownHeight: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  minDropdownWidth: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  icons: PropTypes.shape({
    left: PropTypes.any,
    right: PropTypes.any
  }),
  animate: PropTypes.bool,
  onMenuItemRender: PropTypes.oneOfType([PropTypes.bool, PropTypes.func]),
  dropup: PropTypes.bool,
  flip: PropTypes.bool,
  containerClass: PropTypes.string,
  loading: PropTypes.bool,
  onInputClick: PropTypes.func
}

Select.defaultProps = {
  options: [],
  selected: [],
  multiple: false,
  id: new Date().getTime().toString(),
  onChange: noop,
  searchable: false,
  minLength: 2,
  maxDropdownHeight: MAXIMUM_DROPDOWN_HEIGHT,
  minDropdownWidth: MINIMUM_DROPDOWN_WIDTH,
  inputProps: {},
  filterBy: [],
  labelKey: DEFAULT_LABELKEY,
  closeOnOutsideClick: true,
  width: SELECT_WIDTH,
  height: SELECT_HEIGHT,
  icons: {},
  animate: true,
  onMenuItemRender: false,
  dropup: false,
  flip: false,
  containerClass: '',
  loading: false,
  onInputClick: noop
}

export default Select
