import { ACTUAL_VALUE } from './constants'

export const isString = value => typeof value === 'string'

export const isNumber = value => typeof value === 'number'

export const isObject = value => typeof value === 'object'

export const isFunction = value => typeof value === 'function'

export const stringify = json => JSON.stringify(json)

export const isEqual = (obj1, obj2) => stringify(obj1) === stringify(obj2)

function isMatch(input, string, props) {
  let searchStr = input
  let str = string

  if (!props.caseSensitive) {
    searchStr = searchStr.toLowerCase()
    str = str.toLowerCase()
  }

  return str.indexOf(searchStr) !== -1
}


/**
 * Default algorithm for filtering results.
 */
export function defaultFilterBy(option, props) {
  let { filterBy, labelKey, multiple, selected, value, onMenuItemRender } = props

  // Don't show selected options in the menu for the multi-select case.
  if (multiple && selected.some((o) => isEqual(o, option[ACTUAL_VALUE]))) {
    return false
  }

  let fields = filterBy.slice()

  if (isFunction(onMenuItemRender) && isMatch(value, onMenuItemRender(option), props)) {
    return true
  }

  if (isFunction(labelKey) && isMatch(value, option[labelKey(option)], props)) {
    return true
  }

  if (isString(labelKey)) {
    // Add the `labelKey` field to the list of fields if it isn't already there.
    if (fields.indexOf(labelKey) === -1) {
      fields.unshift(labelKey)
    }
  }

  if (isString(option)) {
    return isMatch(value, option, props)
  }

  return fields.some(field => {
    let text = option[field]

    if (!isString(text)) {
      // Coerce to string since `toString` isn't null-safe.
      text = `${text}`
    }

    return isMatch(value, text, props)
  })
}

export function debounce(callback, interval) {
  let debounceTimeoutId

  return function (...args) {
    clearTimeout(debounceTimeoutId)
    debounceTimeoutId = setTimeout(() => callback.apply(this, args), interval)
  }
}